import TDBRecord as tdbra

def record(user, platform):
    if tdbra.m3u8.checkStatus(user, platform):
        tdbra.logger.info(f"Recording {user} on {platform}")
        newData = tdbra.create_data(user, platform)
        tdbra.data[f"{user}.{platform}"] = newData
        tdbra.m3u8.worker(user, platform)
    else:
        tdbra.logger.error(f"Failed to record {user} on {platform}. Is stream online?")



import TDBRecord as tdbra

from subprocess import Popen, PIPE, DEVNULL
from streamlink import Streamlink
from requests import get
from pathlib import Path
import time

session = Streamlink()
sessionProxy = Streamlink()

urlstore = {
    "twitch": "https://twitch.tv/{user}",
    "afreecatv": "https://play.afreecatv.com/{user}",
    "youtube": "https://www.youtube.com/@{user}/live",
}

def worker(user: str, platform: str) -> None:
    """
    Standalone video downloader
    """
    logger = tdbra.data[f"{user}.{platform}"]["logger"]
    logger.setLevel(tdbra.logger.level)
    downloadPath = tdbra.downloadPath / f"{user}.{platform}"
    downloadPath.mkdir(parents=True, exist_ok=True)
    filename = f"{platform}.{user}.{time.strftime('%y%m%d.%H%M%S')}"

    downloadFullPath = downloadPath / f"{filename}.mp4"
    m3uPlaylistUri = stream(user, platform)
    logger.info("FFMPEG Downloader started.")

    # Downloader
    while True:
        if tdbra.data[f"{user}.{platform}"]["exit"]:
            logger.info("FFMPEG Downloader stopped.")
            del(tdbra.data[f"{user}.{platform}"])
            return

        m3ud = get(m3uPlaylistUri)
        if "twitch-ad-quartile" in m3ud.text:
            logger.debug("Waiting for AD to record...")
            time.sleep(2)
            continue
        break

    logger.info("AD finished. Start recording...")
    tdbra.downloadPath.mkdir(parents=True, exist_ok=True)
    command = [
        tdbra.conf["ffmpeg"],
        "-y",
        "-hide_banner",
        "-loglevel",
        "error",
        "-i",
        m3uPlaylistUri,
        "-c",
        "copy",
        downloadFullPath
    ]
    if tdbra.conf["users"][user]["proxy"]:
        command.insert(1, "-http_proxy")
        command.insert(2, tdbra.conf["proxy"])
    raw = ""
    for i in command:
        raw += f"{i} "
    logger.debug(f"FFMPEG Downloader command: {raw}")
    
    ffmpeg = Popen(command, stdin=PIPE)
    tdbra.data[f"{user}.{platform}"]["ffmpeg"] = ffmpeg
    
    while True:
        if tdbra.data[f"{user}.{platform}"]["exit"]:
            logger.debug("Trying to stop FFMPEG Downloader...")
            ffmpeg.communicate(input=b"q")
            for i in range(6):
                logger.debug(f"Waiting for FFMPEG Downloader to finish... ({i+1}/6)")
                if ffmpeg.poll() is not None:
                    logger.info(f"FFMPEG Downloader finished with status {ffmpeg.poll()}.")
                    del(tdbra.data[f"{user}.{platform}"])
                    return
                time.sleep(1)
            ffmpeg.kill()
            logger.warning("FFMPEG Downloader killed.")
            del(tdbra.data[f"{user}.{platform}"])
            return
        if ffmpeg.poll() is not None:
            logger.info(f"FFMPEG Downloader finished with status {ffmpeg.poll()}.")
            tdbra.data[f"{user}.{platform}"]["end"] = True
            break
        time.sleep(1)
    if tdbra.gc:
        tdbra.autocheck(tdbra.ls, user, platform, downloadFullPath, logger)

    del(tdbra.data[f"{user}.{platform}"])

def stream(user: str, platform: str) -> str:
    if tdbra.conf["users"][user]["proxy"]:
        return sessionProxy.streams(urlstore[platform].format(user=user))["best"].url
    else:    
        return session.streams(urlstore[platform].format(user=user))["best"].url


def checkStatus(user: str, platform: str) -> bool:
    try:
        if tdbra.conf["users"][user]["proxy"]:
            return bool(sessionProxy.streams(urlstore[platform].format(user=user)))
        else:
            return bool(session.streams(urlstore[platform].format(user=user)))
        
    except Exception as e:
        tdbra.logger.error(f"Failed to check status of {user}.{platform}.\nError: {e}")
        return False



import TDBRecord as tdbra

import pathlib
import subprocess
import json
from importlib import import_module

DEFAULT_CONFIG = {
    "version": tdbra.__version__,
    "downloadPath": "./download/",
    "ffmpeg": "ffmpeg", # ffmpeg path, if not set, use PATH ffmpeg
    "proxy": "", # proxy url, if not set, use system proxy. e.g. http://proxy.example.com:8080/
    "users": [
        {
            "name": "ExampleUser1",
            "platform": "twitch | afreecatv | youtube",
            "proxy": True
        },
        {
            "name": "ExampleUser2",
            "platform": "twitch | afreecatv | youtube",
            "proxy": False
        }
    ],
}

PLATFORMS = ["twitch", "afreecatv", "youtube"]

def check_config(conf: dict = {}):
    tdbr = False
    proxy = False
    if not conf:
        if not tdbra.conf:
            tdbra.logger.error("No config found")
            raise ValueError("No config found")
        conf = tdbra.conf
        tdbr = True

    if not "version" in conf:
        tdbra.logger.error("Old config file is not compatible with new version. Please delete old config file and run `tdbrec config` to create new config file.")
        raise ValueError("Old config file is not compatible with new version. Please delete old config file and run `tdbrec config` to create new config file.")
    elif conf["version"].split(".")[0] != tdbra.__version__.split(".")[0]:
        tdbra.logger.error("Old config file is not compatible with new version. Please delete old config file and run `tdbrec config` to create new config file.")
        raise ValueError("Old config file is not compatible with new version. Please delete old config file and run `tdbrec config` to create new config file.")
    elif conf["version"].split(".")[1] != tdbra.__version__.split(".")[1]:
        tdbra.logger.warning("Old config file is not compatible with new version. Please run `tdbrec config` to update config file.")
        tdbra.logger.warning("If you updated to new release, then there may be new config options. Please use `tdbrec config` for new options.")

    if not conf["users"]:
        tdbra.logger.error("No user found in config file")
        raise ValueError("No user found in config file")
    
    for user in conf["users"]:
        if not user["name"]:
            tdbra.logger.error("User name not found in config file")
            raise ValueError("User name not found in config file")
        if not user["platform"]:
            tdbra.logger.error("User platform not found in config file")
            raise ValueError("User platform not found in config file")
        if user["platform"] not in PLATFORMS:
            tdbra.logger.error("User platform not found in config file")
            raise ValueError("User platform not found in config file")
        if "proxy" not in user:
            user["proxy"] = False
        if user["proxy"]:
            proxy = True


    if not conf["downloadPath"]:
        tdbra.error("Download path not found in config file")
        raise ValueError("Download path not found in config file")
    else:
        downloadPath = pathlib.Path(conf["downloadPath"])
        downloadPath.mkdir(parents=True, exist_ok=True)

    if not conf["ffmpeg"]:
        conf["ffmpeg"] = "ffmpeg"
    
    if "gc" in conf:
        try:
            gc = import_module(conf["gc"])
        except ImportError:
            tdbra.logger.error("Cannot import gc module")
            raise ValueError("Cannot import gc module")
        tdbra.gc = gc
        tdbra.ls = gc.ls(tdbra.logger)

    # check ffmpeg is installed
    try:
        subprocess.run([conf["ffmpeg"], "-version"], check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError:
        tdbra.logger.error("ffmpeg not found")
        raise ValueError("ffmpeg not found")
    except FileNotFoundError:
        tdbra.logger.error("ffmpeg not found")
        raise ValueError("ffmpeg not found")
    
    if "proxy" in conf:
        conf["proxy"] = conf["proxy"].strip()
        tdbra.m3u8.sessionProxy.set_option("http-proxy", conf["proxy"])
    elif proxy:
        tdbra.logger.error("Proxy not found in config file")
        raise ValueError("Proxy not found in config file")
    else:
        conf["proxy"] = None

    
    if tdbr: 
        tdbra.conf = conf
        tdbra.downloadPath = downloadPath
    

    return conf, downloadPath

def reload():
    try:
        conf = json.loads(tdbra.confPath.read_text())
    except json.JSONDecodeError:
        tdbra.logger.error("Cannot decode Json file. Config not updated.")
        return False
    except FileNotFoundError:
        tdbra.logger.error("Config file not found. Config not updated.")
        return False
    try:
        conf, downloadPath = check_config(conf)
        tdbra.conf = conf
        tdbra.downloadPath = downloadPath
        tdbra.logger.info("Config updated.")
        return True
    except Exception as e:
        tdbra.logger.error(f"Config check error with <{e}>. Config not updated.")
        return False



import TDBRecord as tdbra

import requests
import pathlib
import logging
import click
import json

__version__ = "1.2.0-pre1"
tdbra.__version__ = __version__


def check_update():
    # check from pypi
    try:
        pypi_version = requests.get("https://pypi.org/pypi/TDBRecord/json").json()["info"]["version"]
    except:
        tdbra.logger.error("Cannot check update")
        return
    if "pre" in __version__: 
        tdbra.logger.info("You are using pre-release version. Please check update manually.")
        tdbra.logger.info("Current version: {version}".format(version=__version__))
        tdbra.logger.info("Latest version: {version}".format(version=pypi_version))
    elif pypi_version != __version__:
        tdbra.logger.info("New version available: {version}".format(version=pypi_version))
        tdbra.logger.info("Please update TDBRecord by 'pip install TDBRecord --upgrade'")
    return



@click.version_option(prog_name="TDBRecord", version=__version__)
@click.group()
def main():
    """
    TDB Record Application for OpenSourceUsers
    """
    pass

@main.command()
@click.option("--debug", is_flag=True, help="Enable debug mode")
@click.option("--config", type=click.Path(exists=True, file_okay=True, dir_okay=False, path_type=pathlib.Path), help="Config file", default="config.json")
@click.option("--logfile", type=click.Path(file_okay=True, dir_okay=False, path_type=pathlib.Path), help="Log file", default="record.log")
def start(debug, config, logfile):
    """
    Start TDBRecord server for multiple users.
    """
    check_update()

    tdbra.confPath = config
    if debug: tdbra.logger.setLevel(logging.DEBUG)
    tdbra.conf = json.loads(tdbra.confPath.read_text())

    tdbra.logFilePath = logfile
    tdbra.logger.addHandler(logging.FileHandler(tdbra.logFilePath))
    tdbra.logger.handlers[1].setFormatter(logging.Formatter('[%(asctime)s] %(levelname)s # %(message)s'))
    tdbra.logger.handlers[1].setLevel(logging.DEBUG)
    logging.getLogger("streamlink.plugins.afreeca").addHandler(logging.FileHandler(tdbra.logFilePath))
    logging.getLogger("streamlink.plugins.afreeca").handlers[1].setFormatter(logging.Formatter('[%(asctime)s] %(levelname)s - afreecatv # %(message)s'))
    logging.getLogger("streamlink.plugins.afreeca").handlers[1].setLevel(logging.INFO)
    tdbra.logger.info("TDBRecord started!")

    tdbra.config.check_config()
    tdbra.command.start()

@main.command()
@click.option("--debug", is_flag=True, help="Enable debug mode")
@click.option("--proxy", help="Proxy Url", default=None)
@click.option("--download-path", type=click.Path(exists=True, file_okay=False, dir_okay=True, path_type=pathlib.Path), help="Download folder", default="download/")
@click.option("--ffmpeg", type=str, help="FFmpeg path", default="ffmpeg")
@click.argument(
    "user",
    type=str,
    required=True,
)
@click.argument(
    "platform",
    type=str,
    required=True,
)
def record(debug, user, platform, proxy, download_path, ffmpeg):
    """
    Standalone record command. Only record one user.
    """

    check_update()

    if debug: tdbra.loglevel = logging.DEBUG
    if proxy: tdbra.conf["proxy"] = proxy
    if ffmpeg: tdbra.conf["ffmpeg"] = ffmpeg
    else: tdbra.conf["ffmpeg"] = "ffmpeg"

    if download_path: tdbra.downloadPath = download_path
    tdbra.command.record(user, platform)

@main.command()
def config():
    """
    Generate config file
    """
    config = pathlib.Path("config.json")
    if config.exists():
        print("Config file already exists")
        return
    config.write_text(json.dumps(tdbra.config.DEFAULT_CONFIG, indent=4))
    tdbra.logger.info("Config file generated")



import TDBRecord.command
import TDBRecord.m3u8
import TDBRecord.config


from prompt_toolkit import PromptSession, print_formatted_text
from streamlink import Streamlink
from pathlib import Path
import logging
__version__ = ""

streamlink = Streamlink()
psession = PromptSession()
input = psession.prompt
print = print_formatted_text
gc = None
# Logging
class PromptHandler(logging.StreamHandler):
    def emit(self, record):
        msg = self.format(record)
        print_formatted_text(msg)

logFilePath = Path("record.log")
logger = logging.getLogger("TDBRecord")
logger.handlers = [PromptHandler()]
logger.handlers[0].setFormatter(logging.Formatter('[%(asctime)s] %(levelname)s # %(message)s'))
lm = "[{user}.{platform}] {msg}"
logger.setLevel(logging.INFO)
logging.getLogger("streamlink.plugins.afreeca").handlers = [PromptHandler()]
logging.getLogger("streamlink.plugins.afreeca").handlers[0].setFormatter(logging.Formatter('[%(asctime)s] %(levelname)s - afreecatv # %(message)s'))

def create_logger(name: str, platform: str) -> logging.Logger:
    ulogger = logging.getLogger(name)
    ulogger.handlers = [PromptHandler(), logging.FileHandler(logFilePath)]
    for handler in ulogger.handlers:
        handler.setFormatter(logging.Formatter('[%(asctime)s] %(levelname)s - [{user}.{platform}] # %(message)s'.format(user=name, platform=platform)))
    ulogger.setLevel(logger.level)
    ulogger.handlers[1].setLevel(logging.DEBUG)
    ulogger.propagate = False
    return ulogger

def create_data(user: str, platform: str, thread = None) -> dict:
    return {
        "exit": False,
        "logger": create_logger(user, platform),
        "thread": thread,
    }

conf = {}
data = {}
confPath = Path("config.json")
downloadPath = Path("download/")